/* Siconos is a program dedicated to modeling, simulation and control
 * of non smooth dynamical systems.
 *
 * Copyright 2020 INRIA.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
*/
#ifndef __LagrangianLinearTIRTest__
#define __LagrangianLinearTIRTest__

#include <cppunit/extensions/HelperMacros.h>
#include "NonSmoothDynamicalSystem.hpp"
#include "LagrangianLinearTIR.hpp"

class LagrangianLinearTIRTest : public CppUnit::TestFixture
{

private:
  /** serialization hooks
  */
  ACCEPT_SERIALIZATION(LagrangianLinearTIRTest);


  // Name of the tests suite
  CPPUNIT_TEST_SUITE(LagrangianLinearTIRTest);

  // tests to be done ...

  CPPUNIT_TEST(testBuildLagrangianLinearTIR1);
  CPPUNIT_TEST(testBuildLagrangianLinearTIR2);
  CPPUNIT_TEST(testBuildLagrangianLinearTIR3);
  CPPUNIT_TEST(testBuildLagrangianLinearTIR4);
  //  CPPUNIT_TEST(testBuildLagrangianLinearTIR5);
  //  CPPUNIT_TEST(testBuildLagrangianLinearTIR6);
  CPPUNIT_TEST(testSetCPtr);
  CPPUNIT_TEST(testSetFPtr);
  CPPUNIT_TEST(testSetEPtr);
  CPPUNIT_TEST(testGetJacPtr);
  CPPUNIT_TEST_SUITE_END();

  // \todo exception test

  void testBuildLagrangianLinearTIR0();
  void testBuildLagrangianLinearTIR1();
  void testBuildLagrangianLinearTIR2();
  void testBuildLagrangianLinearTIR3();
  void testBuildLagrangianLinearTIR4();

  void testSetCPtr();
  void testSetFPtr();
  void testSetEPtr();
  void testGetJacPtr();

  // Members

  SP::SimpleMatrix C, B, F;
  SP::SiconosVector e;
  SP::NonSmoothDynamicalSystem nsds;

public:
  void setUp();
  void tearDown();

};

#endif




